import os
import time
import unittest
import tempfile

from hidb import fileStoreDB

test_data = {
    "test1": "value",
    "test2": 12,
    "test3": [12, 23, 34],
}


class TestFilestoreDB(unittest.TestCase):
    def test_one(self):
        # Test for create, read
        with tempfile.TemporaryDirectory() as tempdir:
            db = fileStoreDB(tempdir)
            db.create("test_data", test_data)
            self.assertEqual(db.read("test_data"), test_data)
            self.assertIsInstance(db.read("test_data"), dict)
            with self.assertRaises(KeyError):
                db.read("test_data2")

    def test_two(self):
        # Test for delete
        with tempfile.TemporaryDirectory() as tempdir:
            db = fileStoreDB(tempdir)
            db.create("test_data", test_data)
            self.assertEqual(db.read("test_data"), test_data)
            db.delete("test_data")
            with self.assertRaises(KeyError):
                db.read("test_data")

    def test_three(self):
        # Test for save data
        with tempfile.TemporaryDirectory() as tempdir:
            db = fileStoreDB(tempdir)
            db.create("test_data", test_data)
            db.saveData("testfile.pk")
            self.assertTrue(os.path.isfile("testfile.pk"))

    def test_four(self):
        # Test for save data and load data
        with tempfile.TemporaryDirectory() as tempdir:
            db = fileStoreDB(tempdir)
            db.create("test_data", test_data)
            db.saveData("testfile.pk")
            self.assertTrue(os.path.isfile("testfile.pk"))
            db2 = fileStoreDB(tempdir)
            db2.loadData("testfile.pk")
            self.assertEqual(db2.read("test_data"), test_data)

    def test_five(self):
        # Test for time to live parameter
        with tempfile.TemporaryDirectory() as tempdir:
            db = fileStoreDB(tempdir)
            db.create("test_data", test_data, 10)
            db.create("test_data2", test_data, 20)
            self.assertEqual(db.read("test_data"), test_data)
            self.assertEqual(db.read("test_data2"), test_data)
            time.sleep(10)
            with self.assertRaises(KeyError):
                db.read("test_data")
            self.assertEqual(db.read("test_data2"), test_data)
            time.sleep(10)
            with self.assertRaises(KeyError):
                db.read("test_data2")



import os
import pickle
from typing import Dict
from datetime import datetime

from .db import DB
from .wrapper import fileWrapper, dataWrapper


class fileStoreDB(DB):
    def __init__(self, location="./"):
        super().__init__()
        self.filestore = fileWrapper()
        self.location = location

    def addNewData(self, key: str, value: Dict, ttl: int):
        self.filestore.keys.add(key)
        self.filestore.data[key] = dataWrapper(value, ttl)

    def showData(self, key: str):
        if self.filestore.data[key].ttl:
            currtime = datetime.today().timestamp()
            datatime = self.filestore.data[key].timestamp
            print(currtime, datatime)
            if currtime - datatime >= self.filestore.data[key].ttl:
                self.delete(key)
                raise KeyError(f"{key} does not exist")

        return self.filestore.data[key].data

    def saveData(self, filename):
        self.filepath = os.path.join(self.location + filename)
        with open(filename, "wb") as outputfile:
            pickle.dump(self.filestore, outputfile, pickle.HIGHEST_PROTOCOL)

    def loadData(self, filename):
        if not os.path.isfile(filename):
            raise FileNotFoundError(f"File not found {os.path.join(self.location, filename)}")
        with open(filename, "rb") as readfile:
            self.filestore = pickle.load(readfile)

    def create(self, key: str, value: Dict, ttl: int = None):
        if len(key) > 32:
            raise KeyError(f"Length of key: {len(key)} exceeds limits should be: 32")
        if key in self.filestore.keys:
            raise KeyError(f"{key} does not exist")
        self.addNewData(key, value, ttl)

    def read(self, key: str):
        if len(key) > 32:
            raise KeyError(f"Length of key: {len(key)} exceeds limits should be: 32")
        if key not in self.filestore.keys:
            raise KeyError(f"{key} does not exist")

        return self.showData(key)

    def delete(self, key: str):
        if len(key) > 32:
            raise KeyError(f"Length of key: {len(key)} exceeds limits should be: 32")
        if key not in self.filestore.keys:
            raise KeyError(f"{key} does not exist")
        self.filestore.keys.remove(key)
        del self.filestore.data[key]



from datetime import datetime


class fileWrapper(object):
    def __init__(self):
        self.data = {}
        self.keys = set()


class dataWrapper:
    def __init__(self, data, ttl):
        self.data = data
        self.timestamp = datetime.today().timestamp()
        self.ttl = ttl



from abc import ABC, abstractmethod


class DB(ABC):
    @abstractmethod
    def create(self):
        """
        Create a key value mapping for data
        """
        raise NotImplementedError

    @abstractmethod
    def read(self):
        """
        Read data given the key
        """
        raise NotImplementedError

    @abstractmethod
    def delete(self):
        """
        Delete the key value mapping for data
        """
        raise NotImplementedError


class KeyError(Exception):
    pass


class ValueError(Exception):
    pass
